"use strict";

window.onload = init;

let gl; // kontekst WebGL

// dane o programie cieniowania
let shaderProgram =
{
    "id" : null,
    "vPosition" : -1,
    "uColor" : null,
    "uModelViewMatrix" : null,
    "uProjectionMatrix" : null
};

// dane o gwiezdzie
let star =
{
    "positionVBO" : null, // VBO zawierajace wspolrzedne
    "fillIndicesVBO" : null, // VBO zawierajace indeksy wierzcholkow scianek
    "fillIndicesCount" : 0, // liczba wierzcholkow do wyrenderowania (wypelnienie)
    "wireframeIndicesVBO" : null, // VBO zawierajace indeksy wierzholkow siatki
    "wireframeIndicesCount" : 0, // liczba wierzcholkow do wyrenderowania (siatka)
    "color" : new Float32Array( [1.0, 1.0, 0.0, 1.0] ),
    "wireframe" : false, // czy rysowac jako siatke (true) lub wypelnienie (false)
    "render" : renderStar // funkcja renderujaca gwiazde
};

// dane o transformacjach
let transformations =
{
    "translationVector" : [0.0, 0.0, 0.0], // wektor translacji
    "rotationAngleZ" : 0.0, // kat obrotu wzgledem osi Z
    "scalingFactors" : [1.0, 1.0, 1.0], // wspolczynniki skali w poszczegolnych kierunkach
    "translationStep" : 0.1, // krok translacji
    "rotationStep" : 10.0, // krok rotacji
    "scaleStep" : 1.1 // wspolczynnik zmiany skali
};

let modelViewMatrix = glMatrix.mat4.create();
let projectionMatrix = glMatrix.mat4.create();

/*
* funkcja inicjuje WebGL oraz ustawia poczatkowe wartosci pewnych ustawien WebGL
*/
function init()
{
    initGL();
    setupShaders();
    setupBuffers();

    initUI();

    document.addEventListener( "keydown", keyboardDown );

    render();
}

/*
* funkcja pobiera kontekst WebGL oraz inicjuje pewne ustawienia WebGL
*/
function initGL()
{
    let canvas = document.getElementById( "gl-canvas" );

    gl = WebGLUtils.setupWebGL( canvas );
    if( !gl )
    {
        alert( "WebGL jest niedostępny!" );
        throw new Error( "WebGL isn't available!" );
    }

    gl.viewport( 0, 0, canvas.width, canvas.height );
    gl.clearColor( 0.7, 0.7, 0.7, 1.0 );

    glMatrix.mat4.ortho( projectionMatrix, -1.0, 1.0, -1.0, 1.0, -1.0, 1.0 );
}

/*
* funkcja inicjuje kontrolki interfejsu uzytkownika
*/
function initUI()
{
    let wireframeCheckbox = document.getElementById( "wireframeEnabled" );
    wireframeCheckbox.addEventListener( "click", wireframeEnabled );
    wireframeCheckbox.checked = star.wireframe;
}

/*
* funkcja zwrotna checkboxa sluzacego do wlaczania/wylaczania rysowania jako siatka
*/
function wireframeEnabled()
{
    star.wireframe = this.checked;
    render();
}

/*
* funkcja wczytuje shadery oraz pobiera lokalizacje atrybutow i zmiennych jednorodnych
*/
function setupShaders()
{
    shaderProgram.id = createProgram( gl, "vertex-shader", "fragment-shader" );

    if( !shaderProgram.id )
        throw new Error( "Shader program creation failed!" );

    shaderProgram.vPosition = gl.getAttribLocation( shaderProgram.id, "vPosition" );

    shaderProgram.uColor = gl.getUniformLocation( shaderProgram.id, "uColor" );
    shaderProgram.uModelViewMatrix = gl.getUniformLocation( shaderProgram.id, "uModelViewMatrix" );
    shaderProgram.uProjectionMatrix = gl.getUniformLocation( shaderProgram.id, "uProjectionMatrix" );
}

/*
* funkcja tworzy obiekty VBO z gwiazda
*/
function setupBuffers()
{
    let starData = generateStar( 5 );

    star.positionVBO = gl.createBuffer();
    gl.bindBuffer( gl.ARRAY_BUFFER, star.positionVBO );
    gl.bufferData( gl.ARRAY_BUFFER, new Float32Array( starData.vertices ), gl.STATIC_DRAW );

    star.fillIndicesVBO = gl.createBuffer();
    gl.bindBuffer( gl.ELEMENT_ARRAY_BUFFER, star.fillIndicesVBO );
    gl.bufferData( gl.ELEMENT_ARRAY_BUFFER, new Uint16Array( starData.indices ), gl.STATIC_DRAW );

    star.fillIndicesCount = starData.indices.length;

    star.wireframeIndicesVBO = gl.createBuffer();
    gl.bindBuffer( gl.ELEMENT_ARRAY_BUFFER, star.wireframeIndicesVBO );
    gl.bufferData( gl.ELEMENT_ARRAY_BUFFER, new Uint16Array( starData.wireframeIndices ), gl.STATIC_DRAW );

    star.wireframeIndicesCount = starData.wireframeIndices.length;
}

/*
* funkcja generuje dane o gwiezdzie z zadana liczba ramion
* arms - liczba ramion
*/
function generateStar(arms)
{
    let vertices = [0.0, 0.0, 0.0, 1.0]; // wsolrzedne kartezjanskie wierzcholkow
    let indices = []; // indeksy wierzcholkow tworzacych scianki

    let innerRadius = 0.5; // promien wewnetrznego okregu
    let outerRadius = 0.9; // promien zewnetrznego okregu
    let offset = Math.PI / arms; // przesuniecie kata pomiedzy wewnetrznymi i zewnetrznymi wierzcholkami

    let outerVertices = [];
    for( let i = 0; i < arms + 1; ++i )
    {
        let theta = i / arms * 2 * Math.PI;
        let x = innerRadius * Math.cos( theta );
        let y = innerRadius * Math.sin( theta );

        vertices.push( x, y, 0.0, 1.0 );

        x = outerRadius * Math.cos( theta + offset );
        y = outerRadius * Math.sin( theta + offset );

        outerVertices.push( x, y, 0.0, 1.0 );
    }

    vertices = vertices.concat( outerVertices );

    let outerIndices = [];
    for( let i = 1; i < arms + 1; ++i )
    {
        indices.push( 0, i, i + 1 ); // indeksy dla trojkatow tworzacych wnetrze gwiazdy
        outerIndices.push( i + 1, i, (arms + 1) + i ); // indeksy dla trojkatow tworzacych ramiona gwiazdy
    }
    indices = indices.concat( outerIndices );

    let wireframeIndices = [];
    for( let i = 0; i < indices.length; i += 3 )
    {
        wireframeIndices.push( indices[i] );
        wireframeIndices.push( indices[i + 1] );

        wireframeIndices.push( indices[i + 1] );
        wireframeIndices.push( indices[i + 2] );

        wireframeIndices.push( indices[i + 2] );
        wireframeIndices.push( indices[i] );
    }

    let result =
    {
        "vertices" : vertices,
        "indices" : indices,
        "wireframeIndices" : wireframeIndices
    };

    return result;
}

/*
* funkcja obslugujaca nacisniecie klawiasza na klawiaturze
*/
function keyboardDown(event)
{
    //console.log( event.keyCode );
    switch( event.keyCode )
    {
        case 107: // +
        case 61:
            transformations.scalingFactors = transformations.scalingFactors.map( scale => scale * transformations.scaleStep );
            break;

        case 109: // -
        case 173:
            transformations.scalingFactors = transformations.scalingFactors.map( scale => scale / transformations.scaleStep );
            break;

        case 38: // strzalka w gore
            transformations.translationVector[1] += transformations.translationStep;
            event.preventDefault();
            break;

        case 40: // strzalka w dol
            transformations.translationVector[1] -= transformations.translationStep;
            event.preventDefault();
            break;

        case 37: // strzalka w lewo
            transformations.translationVector[0] -= transformations.translationStep;
            event.preventDefault();
            break;

        case 39: // strzalka w prawo
            transformations.translationVector[0] += transformations.translationStep;
            event.preventDefault();
            break;

        case 188: // ,
            transformations.rotationAngleZ += transformations.rotationStep;
            break;

        case 190: // .
            transformations.rotationAngleZ -= transformations.rotationStep;
            break;
    }

    render();
}

/*
* funkcja renderuje scene
*/
function render()
{
    gl.clear( gl.COLOR_BUFFER_BIT );

    glMatrix.mat4.identity( modelViewMatrix );
    glMatrix.mat4.scale( modelViewMatrix, modelViewMatrix, transformations.scalingFactors );
    glMatrix.mat4.translate( modelViewMatrix, modelViewMatrix, transformations.translationVector );
    glMatrix.mat4.rotateZ( modelViewMatrix, modelViewMatrix, transformations.rotationAngleZ * Math.PI / 180.0 );

    gl.useProgram( shaderProgram.id );

    gl.uniformMatrix4fv( shaderProgram.uProjectionMatrix, false, projectionMatrix );
    gl.uniformMatrix4fv( shaderProgram.uModelViewMatrix, false, modelViewMatrix );

    star.render( shaderProgram );
}

/*
* funkcja rysuje gwiazde
* shader - program cieniowania, ktory ma byc wykorzystany podczas renderowania
*/
function renderStar(shader)
{
    gl.useProgram( shader.id );

    gl.enableVertexAttribArray( shader.vPosition );

    gl.uniform4fv( shader.uColor, this.color );

    gl.bindBuffer( gl.ARRAY_BUFFER, this.positionVBO );
    gl.vertexAttribPointer( shader.vPosition, 4, gl.FLOAT, false, 0, 0 );

    if( this.wireframe )
    {
        gl.bindBuffer( gl.ELEMENT_ARRAY_BUFFER, this.wireframeIndicesVBO );
        gl.drawElements( gl.LINES, this.wireframeIndicesCount, gl.UNSIGNED_SHORT, 0 );
    }
    else
    {
        gl.bindBuffer( gl.ELEMENT_ARRAY_BUFFER, this.fillIndicesVBO );
        gl.drawElements( gl.TRIANGLES, this.fillIndicesCount, gl.UNSIGNED_SHORT, 0 );
    }
}
