"use strict";

window.onload = init;

let gl;

// dane o programie cieniowania
let shaderProgram =
{
    "id" : null,
    "vPosition" : -1,
    "uColor" : null,
    "uModelMatrix" : null,
    "uViewMatrix" : null,
    "uProjectionMatrix" : null
};

// dane o trojkacie
let triangle =
{
    "positionVBO" : null,
    "noVertices" : 0,
    "color" : new Float32Array( [1.0, 0.0, 0.0, 1.0] )
};

// dane o czworokacie
let quad =
{
    "positionVBO" : null,
    "noVertices" : 0,
    "color" : new Float32Array( [0.0, 1.0, 0.0, 1.0] )
};

let projectionMatrix = glMatrix.mat4.create();
let viewMatrix = glMatrix.mat4.create();

/*
* funkcja inicjuje WebGL oraz ustawia poczatkowe wartosci pewnych ustawien WebGL
*/
function init()
{
    initGL();
    setupShaders();
    setupBuffers();

    initUI();

    render();
}

/*
* funkcja pobiera kontekst WebGL oraz inicjuje pewne ustawienia WebGL
*/
function initGL()
{
    let canvas = document.getElementById( "gl-canvas" );

    gl = WebGLUtils.setupWebGL( canvas );
    if( !gl )
    {
        alert( "WebGL jest niedostępny!" );
        throw new Error( "WebGL isn't available!" );
    }

    gl.viewport( 0, 0, canvas.width, canvas.height );
    gl.clearColor( 0.0, 0.0, 0.0, 1.0 );

    glMatrix.mat4.ortho( projectionMatrix, -1.0, 1.0, -1.0, 1.0, -10.0, 10.0 );
    glMatrix.mat4.lookAt( viewMatrix, [0.0, 0.0, 1.0], [0.0, 0.0, 0.0], [0.0, 1.0, 0.0] );
}

/*
* funkcja inicjuje kontrolki interfejsu uzytkownika
*/
function initUI()
{
    let depthBufferCheckbox = document.getElementById( "depthBufferEnabled" );
    depthBufferCheckbox.addEventListener( "click", depthBufferEnabled );
    depthBufferCheckbox.checked = false;
}

/*
* funkcja zwrotna checkboxa sluzacego do wlaczania/wylaczania bufora glebi
*/
function depthBufferEnabled()
{
    if( this.checked )
        gl.enable( gl.DEPTH_TEST );
    else
        gl.disable( gl.DEPTH_TEST );

    render();
}

/*
* funkcja wczytuje shadery oraz pobiera lokalizacje atrybutow i zmiennych jednorodnych
*/
function setupShaders()
{
    shaderProgram.id = createProgram( gl, "vertex-shader", "fragment-shader" );

    if( !shaderProgram.id )
        throw new Error( "Shader program creation failed!" );

    shaderProgram.vPosition = gl.getAttribLocation( shaderProgram.id, "vPosition" );

    shaderProgram.uColor = gl.getUniformLocation( shaderProgram.id, "uColor" );
    shaderProgram.uModelMatrix = gl.getUniformLocation( shaderProgram.id, "uModelMatrix" );
    shaderProgram.uViewMatrix = gl.getUniformLocation( shaderProgram.id, "uViewMatrix" );
    shaderProgram.uProjectionMatrix = gl.getUniformLocation( shaderProgram.id, "uProjectionMatrix" );
}

/*
* funkcja tworzy obiekty VBO z modelami do wyrenderowania
*/
function setupBuffers()
{
    // trojkat
    let triangleVerts =
    [
        -0.8, 0.0, -0.2, 1.0,
        0.6, -0.7, -0.2, 1.0,
        0.6, 0.7, -0.2, 1.0
    ];

    triangle.positionVBO = gl.createBuffer();
    gl.bindBuffer( gl.ARRAY_BUFFER, triangle.positionVBO );
    gl.bufferData( gl.ARRAY_BUFFER, new Float32Array( triangleVerts ), gl.STATIC_DRAW );

    triangle.noVertices = 3;

    // czworokat
    let quadVerts =
    [
        -0.5, 0.6, -0.5, 1.0,
        -0.7, -0.6, -0.5, 1.0,
        0.5, 0.5, -0.5, 1.0,
        0.8, -0.5, -0.5, 1.0
    ];

    quad.positionVBO = gl.createBuffer();

    gl.bindBuffer( gl.ARRAY_BUFFER, quad.positionVBO );
    gl.bufferData( gl.ARRAY_BUFFER, new Float32Array( quadVerts ), gl.STATIC_DRAW );

    quad.noVertices = 4;
}

/*
* funkcja renderuje scene
*/
function render()
{
    gl.clear( gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT );

    gl.useProgram( shaderProgram.id );

    gl.uniformMatrix4fv( shaderProgram.uProjectionMatrix, false, projectionMatrix );
    gl.uniformMatrix4fv( shaderProgram.uViewMatrix, false, viewMatrix );

    gl.enableVertexAttribArray( shaderProgram.vPosition );

    let triangleModelMatrix = glMatrix.mat4.create();
    gl.uniformMatrix4fv( shaderProgram.uModelMatrix, false, triangleModelMatrix );

    // trojkat
    gl.uniform4fv( shaderProgram.uColor, triangle.color );
    gl.bindBuffer( gl.ARRAY_BUFFER, triangle.positionVBO );
    gl.vertexAttribPointer( shaderProgram.vPosition, 4, gl.FLOAT, false, 0, 0 );

    gl.drawArrays( gl.TRIANGLES, 0, triangle.noVertices );

    let quadModelMatrix = glMatrix.mat4.create();
    gl.uniformMatrix4fv( shaderProgram.uModelMatrix, false, quadModelMatrix );

    // czworokat
    gl.uniform4fv( shaderProgram.uColor, quad.color );
    gl.bindBuffer( gl.ARRAY_BUFFER, quad.positionVBO );
    gl.vertexAttribPointer( shaderProgram.vPosition, 4, gl.FLOAT, false, 0, 0 );

    gl.drawArrays( gl.TRIANGLE_STRIP, 0, quad.noVertices );
}
