"use strict";

window.onload = init;

let gl; // kontekst WebGL

// dane o programie cieniowania
let program =
{
    "id" : null,
    "vPosition" : 0,
    "uModelViewMatrix" : null,
    "uProjectionMatrix" : null,
    "uColor" : null
}

// dane o trojkacie
let triangle =
{
    "positionVBO" : null, // id VBO zawierajacego wspolrzedne
    "color" : new Float32Array( [1.0, 1.0, 0.0, 1.0] ), // kolor
    "noVertices" : 0, // liczba wierzcholkow do wyrenderowania
    "centroid" : [0.0, 0.0, 0.0],
    "rotationAngleZ" : 0.0, // kat obrotu wzgledem osi Z
    "render" : renderModel
};

// dane o kwadracie
let square =
{
    "positionVBO" : null, // id VBO zawierajacego przeplatane wspolrzedne i kolory
    "color" : new Float32Array( [0.0, 1.0, 0.0, 1.0] ), // kolor
    "noVertices" : 0, // liczba wierzcholkow do wyrenderowania
    "translationVector" : [0.0, 0.0, 0.0], //wektor translacji
    "render" : renderModel
};

let modelViewMatrix = glMatrix.mat4.create(); // macierz model-widok
let projectionMatrix = glMatrix.mat4.create(); // macierz rzutowania
let modelViewStack = []; // stos macierzy model-widok

/*
* funkcja inicjuje WebGL oraz ustawia poczatkowe wartosci pewnych ustawien WebGL
*/
function init()
{
    initGL();
    setupShaders();
    setupBuffers();

    window.requestAnimFrame( render );
}

/*
* funkcja pobiera kontekst WebGL oraz inicjuje pewne ustawienia WebGL
*/
function initGL()
{
    let canvas = document.getElementById( "gl-canvas" );

    gl = WebGLUtils.setupWebGL( canvas );
    if( !gl )
    {
        alert( "WebGL jest niedostępny!" );
        throw new Error( "WebGL isn't available!" );
    }

    gl.viewport( 0, 0, canvas.width, canvas.height );
    gl.clearColor( 0.7, 0.7, 0.7, 1.0 );
    gl.enable( gl.DEPTH_TEST );

    glMatrix.mat4.ortho( projectionMatrix, -1.0, 1.0, -1.0, 1.0, -1.0, 1.0 );
}

/*
* funkcja wczytuje shadery oraz pobiera lokalizacje atrybutow i zmiennych jednorodnych
*/
function setupShaders()
{
    program.id = createProgram( gl, "vertex-shader", "fragment-shader" );

    if( !program.id )
        throw new Error( "Shader program creation failed!" );

    program.vPosition = gl.getAttribLocation( program.id, "vPosition" );

    program.uModelViewMatrix = gl.getUniformLocation( program.id, "uModelViewMatrix" );
    program.uProjectionMatrix = gl.getUniformLocation( program.id, "uProjectionMatrix" );
    program.uColor = gl.getUniformLocation( program.id, "uColor" );
}

/*
* funkcja renderuje scene
*/
function render()
{
    gl.clear( gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT );

    glMatrix.mat4.identity( modelViewMatrix );

    gl.useProgram( program.id );

    gl.enableVertexAttribArray( program.vPosition );

    gl.uniformMatrix4fv( program.uProjectionMatrix, false, projectionMatrix );

    // trojkat
    modelViewStack.push( glMatrix.mat4.clone(modelViewMatrix) );

    glMatrix.mat4.translate( modelViewMatrix, modelViewMatrix, triangle.centroid );
    glMatrix.mat4.rotateZ( modelViewMatrix, modelViewMatrix, triangle.rotationAngleZ * Math.PI / 180.0 );
    glMatrix.mat4.translate( modelViewMatrix, modelViewMatrix, triangle.centroid.map( v => -v) );

    gl.uniformMatrix4fv( program.uModelViewMatrix, false, modelViewMatrix );
    gl.uniform4fv( program.uColor, triangle.color );

    triangle.render();

    modelViewMatrix = modelViewStack.pop();

    // kwadrat
    modelViewStack.push( glMatrix.mat4.clone(modelViewMatrix) );

    glMatrix.mat4.translate( modelViewMatrix, modelViewMatrix, square.translationVector );

    gl.uniformMatrix4fv( program.uModelViewMatrix, false, modelViewMatrix );
    gl.uniform4fv( program.uColor, square.color );

    square.render();

    modelViewMatrix = modelViewStack.pop();

    update();

    window.requestAnimFrame( render );
}

/*
* funkcja renderujaca model
*/
function renderModel()
{
    gl.bindBuffer( gl.ARRAY_BUFFER, this.positionVBO );
    gl.vertexAttribPointer( program.vPosition, 4, gl.FLOAT, false, 0, 0 );

    gl.drawArrays( gl.TRIANGLE_STRIP, 0, this.noVertices );
}

/*
* funkcja aktualizuje parametry obiektow dla nastepnej klatki animacji
*/
function update()
{
    triangle.rotationAngleZ += 5.0;
    if( triangle.rotationAngleZ > 360.0 )
        triangle.rotationAngleZ -= 360.0;

    let y = square.translationVector[1] + 0.01;
    if( y > 1.5 )
        y = -1.5;
    square.translationVector[1] = y;
}

/*
* funkcja tworzy obiekty VBO z modelami do wyrenderowania
*/
function setupBuffers()
{
    // trojkat
    let triangleVertices =
    [
        0.1, 0.0, 0.0, 1.0,
		0.9, -0.5, 0.0, 1.0,
		0.9, 0.5, 0.0, 1.0
    ];

    triangle.positionVBO = gl.createBuffer();
    gl.bindBuffer( gl.ARRAY_BUFFER, triangle.positionVBO );
    gl.bufferData( gl.ARRAY_BUFFER, new Float32Array( triangleVertices ), gl.STATIC_DRAW );

    triangle.centroid[0] = (triangleVertices[0] + triangleVertices[4] + triangleVertices[8]) / 3.0;
    triangle.centroid[1] = (triangleVertices[1] + triangleVertices[5] + triangleVertices[9]) / 3.0;
    triangle.centroid[2] = (triangleVertices[2] + triangleVertices[6] + triangleVertices[10]) / 3.0;

    triangle.noVertices = 3;

    // kwadrat
    let squareVertices =
	[
		-0.9, 0.4, 0.0, 1.0,
		-0.9, -0.4, 0.0, 1.0,
		-0.1, 0.4, 0.0, 1.0,
		-0.1, -0.4, 0.0, 1.0,
	];

    square.positionVBO = gl.createBuffer();

    gl.bindBuffer( gl.ARRAY_BUFFER, square.positionVBO );
    gl.bufferData( gl.ARRAY_BUFFER, new Float32Array( squareVertices ), gl.STATIC_DRAW );

    square.noVertices = 4;
}
